////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.send.flickr;

import java.io.InputStream;
import org.watto.android.Language;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.ContentResolver;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Binder;
import android.os.IBinder;


/***********************************************************************************************
A <code>Service</code> for showing <code>Notification</code>s
***********************************************************************************************/
public class FlickrUploadService extends Service {

  NotificationManager notificationManager;

  /** The ID for all Flickr-related notifications **/
  public static int FLICKR_NOTIFICATION = 0;

  /** A notification that says that the photo upload has started **/
  public static int NOTIFICATION_UPLOAD_STARTED = 1;
  /** A notification that says that the photo was successfully uploaded **/
  public static int NOTIFICATION_UPLOAD_SUCCESS = 2;
  /** A notification that says that there was an error uploading the photo **/
  public static int NOTIFICATION_UPLOAD_FAILURE = 3;

  /** The Binder object that receives interactions from other clients **/
  private final IBinder binder = new FlickrUploadServiceBinder();

  /** The Intent that contains the content to upload **/
  Intent uploadIntent;


  //int progress = 0;
  //Notification notification;
  //int maximum = 0;

  /***********************************************************************************************
  Constructor
  ***********************************************************************************************/
  public FlickrUploadService(){
    super();
  }


  /***********************************************************************************************
  Gets the <code>IBinder</code> to this <code>Service</code>
  @param intent the <code>Intent</code> that is being bound
  @return the <i>binder</i>
  ***********************************************************************************************/
  public IBinder onBind(Intent intent){
    return binder;
  }


  /***********************************************************************************************
  Creates the <code>Service</code>
  ***********************************************************************************************/
  public void onCreate(){
    notificationManager = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
  }


  /***********************************************************************************************
  Finishes the <code>Service</code>
  ***********************************************************************************************/
  public void onDestroy(){
    //notificationManager.cancel(FLICKR_NOTIFICATION);
  }


  /***********************************************************************************************
  Runs the <code>Service</code>
  ***********************************************************************************************/
  public int onStartCommand(Intent intent,int flags,int startId){

    uploadIntent = intent;

    // Run all the processing in a separate Thread so that the Service returns without timing out
    runUploadPhotoTask();

    // The service will be killed when it has finished - no point leaving it hang around
    return START_NOT_STICKY;
  }


  /***********************************************************************************************
  Shows a <code>Notification</code> whilever this <code>Service</code> is running
  @param notificationType the type of notification to display
  ***********************************************************************************************/
  public Notification showNotification(int notificationType){

    // Prepare the notification properties
    int icon = -1;
    String tickerText = "";
    String title = "";
    String longText = "";
    long when = System.currentTimeMillis();
    Intent notificationIntent = null;

    // Load the notification properties based on the notification type
    if (notificationType == NOTIFICATION_UPLOAD_STARTED) {
      icon = android.R.drawable.stat_sys_upload;
      tickerText = Language.get(R.string.notification_upload_started_ticker_text);
      title = Language.get(R.string.notification_upload_started_title);
      longText = Language.get(R.string.notification_upload_started_long_text);
      // TODO Change this Intent to a class for handling the display of current progress, or something like that
      // Ditto for all the other 'else-if' statements below.
      //notificationIntent = new Intent(this,FlickrUploader.class);
    }
    else if (notificationType == NOTIFICATION_UPLOAD_SUCCESS) {
      icon = android.R.drawable.stat_sys_upload_done;
      tickerText = Language.get(R.string.notification_upload_success_ticker_text);
      title = Language.get(R.string.notification_upload_success_title);
      longText = Language.get(R.string.notification_upload_success_long_text);
      //notificationIntent = new Intent(this,FlickrUploader.class);
    }
    else if (notificationType == NOTIFICATION_UPLOAD_FAILURE) {
      icon = android.R.drawable.stat_notify_error;
      tickerText = Language.get(R.string.notification_upload_failure_ticker_text);
      title = Language.get(R.string.notification_upload_failure_title);
      longText = Language.get(R.string.notification_upload_failure_long_text);
      //notificationIntent = new Intent(this,FlickrUploader.class);
    }
    else {
      return null;
    }

    // Cancel any current Flickr notifications
    notificationManager.cancel(FLICKR_NOTIFICATION);

    // Show the new notification
    Notification notification = new Notification(icon,tickerText,when);
    /*
    if (notificationType == NOTIFICATION_UPLOAD_STARTED) {
      RemoteViews contentView = new RemoteViews(getPackageName(),R.layout.upload_notification);
      contentView.setTextViewText(R.id.tf_upload_notification_message,longText);
      notification.contentView = contentView;
      PendingIntent contentIntent = PendingIntent.getActivity(this,0,notificationIntent,0);
      notification.contentIntent = contentIntent;
      notificationManager.notify(FLICKR_NOTIFICATION,notification);
    }
    else {
    */

    PendingIntent contentIntent = null;
    try {
      contentIntent = PendingIntent.getActivity(this,0,notificationIntent,0);
    }
    catch (IllegalArgumentException e) {
      // v1.01 - Bug Fix - 1 person got an exception due to notificationIntent being null 
    }

    notification.setLatestEventInfo(getApplicationContext(),title,longText,contentIntent);
    notificationManager.notify(FLICKR_NOTIFICATION,notification);

    //}

    return notification;
  }

  /***********************************************************************************************
  Binder class for remote access
  ***********************************************************************************************/
  public class FlickrUploadServiceBinder extends Binder {

    FlickrUploadService getService(){
      return FlickrUploadService.this;
    }
  }

  /***********************************************************************************************
  A background Thread for uploading the photo (so that the Service doesn't time out)
   ***********************************************************************************************/
  protected class UploadPhotoTask extends AsyncTask<Object,Object,Object> {

    /** the thumbnail image **/
    Bitmap thumbnailBitmap = null;


    /***********************************************************************************************
    Uploads the photo to Flickr in the background
    @param arguments the arguments passed to the background task
    @return the result of the background operation
     ***********************************************************************************************/
    protected Object doInBackground(Object ... arguments){

      // Get all the upload information and the image data from the Intent
      Uri photoUrl = (Uri)uploadIntent.getParcelableExtra("photoUrl");
      String title = uploadIntent.getStringExtra("title");
      String description = uploadIntent.getStringExtra("description");
      String tags = uploadIntent.getStringExtra("tags");
      String visibility = uploadIntent.getStringExtra("visibility");
      String safety = uploadIntent.getStringExtra("safety");
      String contentType = uploadIntent.getStringExtra("contentType");

      byte[] photoData = null;

      // Get the photo data
      try {
        ContentResolver cr = getContentResolver();
        InputStream is = cr.openInputStream(photoUrl);
        photoData = FlickrUploader.readLocalPhoto(is);
      }
      catch (Throwable t) {
        // Error getting the photo data
      }

      // Alert that the upload has started
      showNotification(NOTIFICATION_UPLOAD_STARTED);
      //notification.contentView.setProgressBar(R.id.progress_upload_notification_progress,photoData.length,0,false);
      //maximum = photoData.length;

      // Run the upload
      boolean success = FlickrQuery.uploadPhoto(photoData,title,description,tags,visibility,safety,contentType);

      // Alert that the upload was successful or had errors
      if (success) {
        showNotification(NOTIFICATION_UPLOAD_SUCCESS);
      }
      else {
        showNotification(NOTIFICATION_UPLOAD_FAILURE);
      }

      stopSelf();

      return true;
    }


    /***********************************************************************************************
    Does nothing
    @param result the result of the background operation
     ***********************************************************************************************/
    protected void onPostExecute(Object result){
      // Nothing
    }

  }


  /***********************************************************************************************
  Runs a <code>AsyncTask</code> that uploads the photo to Flickr
  ***********************************************************************************************/
  public void runUploadPhotoTask(){
    new UploadPhotoTask().execute();
  }

}
