////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.send.flickr;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import org.watto.android.Language;
import org.watto.android.Settings;
import org.watto.android.WSAboutScreen;
import org.watto.android.WSSettingsChanger;
import org.watto.android.component.WSRadioButtonDropdown;
import org.watto.android.event.WSViewClickableInterface;
import org.watto.android.event.listener.WSViewClickableListener;
import org.watto.program.android.send.flickr.authenticator.AuthenticatorActivity;
import org.watto.program.android.send.flickr.authenticator.FlickrAuthenticator;
import android.app.Activity;
import android.content.ContentResolver;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;


/***********************************************************************************************
An activity that allows the user to upload an image to Flickr
***********************************************************************************************/
public class FlickrUploader extends Activity implements WSViewClickableInterface {

  Uri photoUrl = null;
  
  /** The data of the photo to upload **/
  //byte[] photoData = new byte[0];
  /** Whether the thumbnail of the photo has been generated yet or not **/
  boolean thumbnailLoaded = false;

  /** Indicates that this is the first time the application has run, so don't finish() in onResume() **/
  boolean firstTime = false;

  /** The Intent used to create the Activity in the onCreate method **/
  Intent loadActivityIntent = null;

  /** The action to perform when no account has been created yet, or to renew an authentication **/
  public static String ACTION_NORMAL_AUTHENTICATION = "authoriseAccount";
  /** The action to perform when changing to a different Flickr account **/
  public static String ACTION_CHANGE_USER = "changeUser";

  /** The action to perform when no account has been created yet, or to renew an authentication **/
  public static int ACTIVITY_AUTHENTICATOR_ACTIVITY = 1;

  /** The Service that controls <code>Notification</code>s **/
  FlickrUploadService notificationService = new FlickrUploadService();


  ///** A notification that says that the photo upload has started **/
  //public static int NOTIFICATION_UPLOAD_STARTED=1;
  ///** A notification that says that the photo was successfully uploaded **/
  //public static int NOTIFICATION_UPLOAD_SUCCESS=2;
  ///** A notification that says that there was an error uploading the photo **/
  //public static int NOTIFICATION_UPLOAD_FAILURE=3;

  /***********************************************************************************************
  Gets a <code>Bitmap</code> for a <code>byte</code> array
  @param photoBytes the <code>byte</code> array of photo data
  @return the <code>Bitmap</code>
  ***********************************************************************************************/
  public Bitmap getBitmap(byte[] photoBytes){
    try {
      if (photoBytes != null && photoBytes.length > 0) {
        Bitmap loadedJpg = BitmapFactory.decodeByteArray(photoBytes,0,photoBytes.length);
        return loadedJpg;
      }

    }
    catch (Throwable t) {}
    return null;
  }


  /***********************************************************************************************
  Gets the value of a <code>EditText</code>
  @param resourceId the resource ID of the <code>EditText</code> to get the value of
  @return the value of the <code>EditText</code>
  ***********************************************************************************************/
  public String getEditTextValue(int resourceId){
    EditText field = (EditText)findViewById(resourceId);
    if (field != null) {
      return field.getText().toString();
    }
    return null;
  }


  /***********************************************************************************************
  Gets the value of a <code>WSRadioButtonDropdown</code>
  @param resourceId the resource ID of the <code>WSRadioButtonDropdown</code> to get the value of
  @return the value of the <code>WSRadioButtonDropdown</code>
  ***********************************************************************************************/
  public String getRadioButtonDropdownValue(int resourceId){
    WSRadioButtonDropdown field = (WSRadioButtonDropdown)findViewById(resourceId);
    if (field != null) {
      return field.getSelectedValue();
    }
    return null;
  }


  /***********************************************************************************************
  Processes events that are triggered when a sub-<code>Activity</code> is <code>finish()</code>ed
  @param requestCode the code of the <code>Activity</code> that has finished
  @param resultCode the result of the <code>Activity</code>
  @param intent the data returned from the Activity
  ***********************************************************************************************/
  public void onActivityResult(int requestCode,int resultCode,Intent intent){
    super.onActivityResult(requestCode,resultCode,intent);

    if (requestCode == ACTIVITY_AUTHENTICATOR_ACTIVITY) {
      if (resultCode == RESULT_OK) {
        // The Flickr account was created, or the user was changed, so (re)load the username/thumbnail
        runLoadUsernameTask();
        runLoadPhotoThumbnailTask();
      }
      else if (resultCode == RESULT_CANCELED) {
        // Clicked the Back button, or an error.

        // Check if the user has a valid Flickr access token.
        if (!FlickrAuthenticator.credentialsKnown()) {
          // Invalid or missing auth token, so they hit the back button when running the first time.
          finish();
        }
      }
    }
  }


  /***********************************************************************************************
  Performs an action when a single-click event is triggered on a <code>View</code>
  @param source the <code>View</code> that triggered the event
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onClick(View source){
    int sourceId = source.getId();

    if (sourceId == R.id.button_upload) {

      String title = getEditTextValue(R.id.tf_title);
      String description = getEditTextValue(R.id.tf_description);
      String tags = getEditTextValue(R.id.tf_tags);
      String visibility = getRadioButtonDropdownValue(R.id.dropdown_visibility);
      String safety = getRadioButtonDropdownValue(R.id.dropdown_safety);
      String contentType = getRadioButtonDropdownValue(R.id.dropdown_content_type);

      // Run the upload service (in the background)
      runUploadService(photoUrl,title,description,tags,visibility,safety,contentType);

      // Close the application
      finish();

    }
    else if (sourceId == R.id.button_change_account) {
      showAuthenticatorActivity(ACTION_CHANGE_USER);
    }
    else {
      return false;
    }
    return true;
  }


  /***********************************************************************************************
  Creates the <code>Activity</code> from the XML file
  @param savedInstanceState the saved state of the <code>Activity</code>
  ***********************************************************************************************/
  public void onCreate(Bundle savedInstanceState){
    super.onCreate(savedInstanceState);

    // TODO REMOVE THIS LINE!!!
    //android.os.Debug.waitForDebugger();

    // Prepare the Settings (and Language implicitly)
    Settings.setApplication(this);

    // Remember the Intent so we can load the image after logging in for the first time
    this.loadActivityIntent = getIntent();

    // Check if the user has a valid Flickr access token.
    if (!FlickrAuthenticator.credentialsKnown()) {
      // Invalid or missing auth token.
      // So, take them to the auth page first.
      showAuthenticatorActivity(ACTION_NORMAL_AUTHENTICATION);
      firstTime = true;
    }

    // Load the main interface.
    // Note: If this is the first time accessing the application, the user is taken to the
    // AccountAuthenticator first to sign in, then is returned to this interface. However, 
    // we must load the interface here so it has something to come back to instead of exiting.

    // Creates the view from the XML file
    setContentView(R.layout.flickr_uploader);

    // Sets the view click listeners
    WSViewClickableListener viewClickListener = new WSViewClickableListener(this);
    ((Button)findViewById(R.id.button_upload)).setOnClickListener(viewClickListener);
    ((Button)findViewById(R.id.button_change_account)).setOnClickListener(viewClickListener);

    // Load the username from the settings and show it as the default until the real username is loaded
    String previousUsername = Settings.getString("username");
    if (previousUsername != null && !previousUsername.equals("")) {
      ((TextView)findViewById(R.id.label_account_name)).setText(previousUsername);
    }
  }


  /***********************************************************************************************
  Creates the <code>Menu</code> from the XML file when the Options button is pressed
  @param menu the <code>Menu</code> <code>View</code> to populate with menu items
  @return <b>true</b> to display the menu, <br />
          <b>false</b> to keep the menu hidden.
  ***********************************************************************************************/
  public boolean onCreateOptionsMenu(Menu menu){
    MenuInflater inflater = getMenuInflater();
    inflater.inflate(R.menu.flickr_uploader,menu);
    return true;
  }


  /***********************************************************************************************
  Performs an action when a <code>MenuItem</code> is selected in the Options menu
  @param item the <code>MenuItem</code> that was selected
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onOptionsItemSelected(MenuItem item){
    int id = item.getItemId();

    if (id == R.id.menu_settings) {
      showSettings();
    }
    else if (id == R.id.menu_about) {
      showAbout();
    }
    else {
      return false;
    }
    return true;
  }


  /***********************************************************************************************
  Resumes the <code>Activity</code> (ie. the activity is visible again)
  ***********************************************************************************************/
  public void onResume(){
    super.onResume();

    if (firstTime) {
      // Stop right here - we're showing the AuthenticatorActivity so don't want to continue loading
      // this interface or running the LoadXXXTasks() yet. Triggered in onActivityResult() instead.
      return;
    }

    // Check if the user has a valid Flickr access token.
    if (!FlickrAuthenticator.credentialsKnown()) {
      // nope, the user must have hit the Back button instead of signing in,
      // so just close the application
      finish();
    }

    // Load the username and thumbnail in the background
    runLoadUsernameTask();
    runLoadPhotoThumbnailTask();
  }


  /***********************************************************************************************
  Reads a photo from a local <code>dataStream</code>.
  @param dataStream the local stream to the photo
  @return the <code>byte</code> array of photo data
  ***********************************************************************************************/
  public static byte[] readLocalPhoto(InputStream dataStream){
    try {
      if (dataStream != null && dataStream.available() >= 0) {
        // Read the photo from the local filesystem into a byte[]
        /*
        byte[] photoBytes = new byte[1000];

        int readSize = 0;
        int receivedSize = dataStream.read(photoBytes,readSize,1000);

        while (receivedSize > 0) {
          // there is more data, so increase the array by another 1000 bytes and repeat the read
          readSize += receivedSize;
          photoBytes = ArrayResizer.resize(photoBytes,readSize + 1000);
          receivedSize = dataStream.read(photoBytes,readSize,1000);
        }

        dataStream.close();

        // resize to the actual size of the content
        photoBytes = ArrayResizer.resize(photoBytes,readSize);
        return photoBytes;
        */
        
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        
        int available = dataStream.available();

        while (available > 0) {
          byte[] readBytes = new byte[available];
          dataStream.read(readBytes);
          outStream.write(readBytes);
          available = dataStream.available();
        }
        
        dataStream.close();

        outStream.close();
        return outStream.toByteArray();

      }

    }
    catch (Throwable t) {
      // Error downloading the photo
    }
    return new byte[0];
  }


  /***********************************************************************************************
  Resizes a photo in a <code>byte</code> array
  @param photoBytes the <code>byte</code> array of photo data
  @return the resized photo as a <code>Bitmap</code>
  ***********************************************************************************************/
  public Bitmap resizePhoto(byte[] photoBytes){
    try {
      if (photoBytes != null && photoBytes.length > 0) {

        /*
        // Now check that the bytes are a valid image file
        Bitmap loadedJpg = BitmapFactory.decodeByteArray(photoBytes,0,photoBytes.length);
        if (loadedJpg != null) {
          // if the bitmap is larger than 150x150, resize it...
          int width = loadedJpg.getWidth();
          int height = loadedJpg.getHeight();
          if (height > 100) {
            double resizeFactor = 100d / ((double)height);
            height = 100;
            width = (int)(((double)width) * resizeFactor); // get the scaled width in ratio

            // resize it
            loadedJpg = Bitmap.createScaledBitmap(loadedJpg,width,height,true);
            
            BitmapFactory.Options options = new BitmapFactory.Options();
            options.inSampleSize = options.outHeight/100;
            Bitmap bitmap = BitmapFactory.decodeStream(is,null,options);
            
            // get the new bytes for the resized image
            ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
            loadedJpg.compress(Bitmap.CompressFormat.JPEG,75,byteStream);
            photoBytes = byteStream.toByteArray();
            
          }
        }
        */
        
        
        BitmapFactory.Options options = new BitmapFactory.Options();
        BitmapFactory.decodeByteArray(photoBytes,0,photoBytes.length,options);
        int photoHeight = options.outHeight;
        
        options.inSampleSize = (int)(((double)photoHeight)/100)+1;
        //options.inDither = false;
        Bitmap loadedJpg = BitmapFactory.decodeByteArray(photoBytes,0,photoBytes.length,options);

        return loadedJpg;

      }

    }
    catch (Throwable t) {
      // Error downloading the photo
    }
    return null;
  }


  /***********************************************************************************************
  Runs a <code>AsyncTask</code> that generates a thumbnail of the chosen photo in the background
  ***********************************************************************************************/
  public void runLoadPhotoThumbnailTask(){
    if (!thumbnailLoaded) {
      new LoadPhotoThumbnailTask().execute();
    }
  }


  /***********************************************************************************************
  Runs a <code>AsyncTask</code> that loads the Flickr Username in the background
  ***********************************************************************************************/
  public void runLoadUsernameTask(){
    new LoadUsernameTask().execute();
  }


  /***********************************************************************************************
  Runs the <code>FlickrUploadService</code>
  ***********************************************************************************************/
  public void runUploadService(Uri photoUrl,String title,String description,String tags,String visibility,String safety,String contentType){

    // Build an intent to pass all the photo information to the Service
    Intent uploadIntent = new Intent(getBaseContext(),FlickrUploadService.class);
    uploadIntent.putExtra("photoUrl",photoUrl);
    uploadIntent.putExtra("title",title);
    uploadIntent.putExtra("description",description);
    uploadIntent.putExtra("tags",tags);
    uploadIntent.putExtra("visibility",visibility);
    uploadIntent.putExtra("safety",safety);
    uploadIntent.putExtra("contentType",contentType);

    // Notify the user that the upload has started
    showToast(FlickrUploadService.NOTIFICATION_UPLOAD_STARTED);

    // Run the Service
    startService(uploadIntent);

  }


  /***********************************************************************************************
  Sets the value of the Username field
  @param username the username
  ***********************************************************************************************/
  public void setUsername(String username){
    // Remember the username for loading into the box by default next time
    Settings.set("username",username);

    // Display the username
    ((TextView)findViewById(R.id.label_account_name)).setText(username);
  }


  /***********************************************************************************************
  Shows the about <code>Activity</code>
  ***********************************************************************************************/
  public void showAbout(){
    Intent aboutActivity = new Intent(getBaseContext(),WSAboutScreen.class);
    startActivity(aboutActivity);
  }


  /***********************************************************************************************
  Displays the <code>AuthenticatorActivity</code>, where the user can sign in to Flickr and
  authorize the application.
  ***********************************************************************************************/
  public void showAuthenticatorActivity(String authenticationAction){
    Intent authenticatorActivity = new Intent(getBaseContext(),AuthenticatorActivity.class);
    authenticatorActivity.putExtra("action",authenticationAction);
    startActivityForResult(authenticatorActivity,ACTIVITY_AUTHENTICATOR_ACTIVITY);
  }


  /***********************************************************************************************
  Shows the settings <code>Activity</code>
  ***********************************************************************************************/
  public void showSettings(){
    Intent settingsActivity = new Intent(getBaseContext(),WSSettingsChanger.class);
    startActivity(settingsActivity);
  }


  /***********************************************************************************************
  Shows a <code>Toast</code> message
  @param notificationType the type of toast to display
  ***********************************************************************************************/
  public void showToast(int notificationType){

    String message = "";

    // Load the toast properties based on the notification type
    if (notificationType == FlickrUploadService.NOTIFICATION_UPLOAD_STARTED) {
      message = Language.get(R.string.notification_upload_started_toast);
    }
    else {
      return;
    }

    Toast.makeText(this,message,Toast.LENGTH_LONG).show();
  }

  /***********************************************************************************************
  A background Thread for generating a thumbnail of the photo and displaying it on the GUI
   ***********************************************************************************************/
  protected class LoadPhotoThumbnailTask extends AsyncTask<Object,Object,Object> {

    /** the thumbnail image **/
    Bitmap thumbnailBitmap = null;


    /***********************************************************************************************
    Runs the task in the background
    @param arguments the arguments passed to the background task
    @return the result of the background operation
     ***********************************************************************************************/
    protected Object doInBackground(Object ... arguments){
      // See if we're calling this from a "Share via Flickr" menu
      Intent intent = loadActivityIntent;
      if (intent != null) {
        String action = intent.getAction();
        if (action != null && action.equals(Intent.ACTION_SEND)) {
          Bundle extras = intent.getExtras();
          if (extras.containsKey(Intent.EXTRA_STREAM)) {
            // Yep, it is, so grab the picture that we're trying to upload

            try {
              // Get the URI resource path of the image to upload
              photoUrl = (Uri)extras.getParcelable(Intent.EXTRA_STREAM);

              // Open a ContentResolver for reading the image from the URI
              ContentResolver cr = getContentResolver();
              InputStream is = cr.openInputStream(photoUrl);

              // Get the bytes of the file, and save them to a global variable
              byte[] photoData = readLocalPhoto(is);
              
              // Prepare the thumbnail Bitmap object 
              thumbnailBitmap = resizePhoto(photoData);

            }
            catch (Throwable t) {}

          }
        }
      }
      return true;
    }


    /***********************************************************************************************
    Updates the GUI after the task has finished running
    @param result the result of the background operation
     ***********************************************************************************************/
    protected void onPostExecute(Object result){
      thumbnailLoaded = true;

      ImageView thumbnailHolder = (ImageView)findViewById(R.id.image_thumbnail);

      if (thumbnailBitmap != null) {
        thumbnailHolder.setImageBitmap(thumbnailBitmap);
      }
    }

  }

  /***********************************************************************************************
  A background Thread for loading the Flickr Username into the username field on the GUI
   ***********************************************************************************************/
  protected class LoadUsernameTask extends AsyncTask<Object,Object,Object> {

    /** the username **/
    String username;


    /***********************************************************************************************
    Runs the task in the background
    @param arguments the arguments passed to the background task
    @return the result of the background operation
     ***********************************************************************************************/
    protected Object doInBackground(Object ... arguments){
      username = FlickrQuery.getUsername();
      return true;
    }


    /***********************************************************************************************
    Updates the GUI after the task has finished running
    @param result the result of the background operation
     ***********************************************************************************************/
    protected void onPostExecute(Object result){
      setUsername(username);
    }

  }

}
