////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.send.flickr.authenticator;

import org.watto.android.component.WSAndroidPopup;
import org.watto.android.event.WSViewClickableInterface;
import org.watto.android.event.listener.WSViewClickableListener;
import org.watto.program.android.send.flickr.R;
import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;


/***********************************************************************************************
Activity which displays login screen to the user
***********************************************************************************************/
public class AuthenticatorActivity extends Activity implements WSViewClickableInterface {

  public int ACTIVITY_CALL_BROWSER = 2;


  /***********************************************************************************************
  Processes events that are triggered when a sub-<code>Activity</code> is <code>finish()</code>ed
  @param requestCode the code of the <code>Activity</code> that has finished
  @param resultCode the result of the <code>Activity</code>
  @param data the data returned from the Activity
  ***********************************************************************************************/
  public void onActivityResult(int requestCode,int resultCode,Intent data){
    super.onActivityResult(requestCode,resultCode,data);

    if (requestCode == ACTIVITY_CALL_BROWSER) {
      // Show the Verifying and Syncing screen, so the user is kept informed
      setContentView(R.layout.verifying);

      // Open a new Thread to run the Callback processing (takes it out of the GUI thread)
      //new Thread(new AuthenticatorCallbackThread(this,data)).start();
      
      // We want to stay in the GUI thread so that we return to the main FlickrUploader interface
      // once the finish() is called. Thats why its not in a different thread like above!
      onNewIntent(data);
    }
  }


  /***********************************************************************************************
  Performs an action when a single-click event is triggered on a <code>View</code>
  @param source the <code>View</code> that triggered the event
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onClick(View source){
    int sourceId = source.getId();
    if (sourceId == R.id.button_authorise_account) {
      // Open the URL for adding an account
      Intent callBrowser = FlickrAuthenticator.getAuthenticationIntent(this);

      if (callBrowser == null) {
        // Internet Access Error
        WSAndroidPopup.showError(this,R.string.error_connection_failure);
        finish();
      }

      startActivityForResult(callBrowser,ACTIVITY_CALL_BROWSER);
    }
    else if (sourceId == R.id.button_update_credentials) {
      // Open the URL for updating the credentials for an account
      Intent callBrowser = FlickrAuthenticator.getAuthenticationIntent(this);

      if (callBrowser == null) {
        // Internet Access Error
        WSAndroidPopup.showError(this,R.string.error_connection_failure);
        finish();
      }

      startActivityForResult(callBrowser,ACTIVITY_CALL_BROWSER);
    }
    if (sourceId == R.id.button_change_user_authorise) {
      // Open the URL for adding an account
      Intent callBrowser = FlickrAuthenticator.getAuthenticationIntent(this);

      if (callBrowser == null) {
        // Internet Access Error
        WSAndroidPopup.showError(this,R.string.error_connection_failure);
        finish();
      }

      startActivityForResult(callBrowser,ACTIVITY_CALL_BROWSER);
    }
    else if (sourceId == R.id.button_change_user_cancel) {
      // Didn't actually want to change the user account, so just back out
      finish();
      }
    else {
      return false;
    }
    return true;
  }


  /***********************************************************************************************
  Creates the <code>Activity</code>
  ***********************************************************************************************/
  public void onCreate(Bundle bundle){
    super.onCreate(bundle);
    
    setResult(RESULT_CANCELED);

    // Get the Intent that started this activity, and the action of the Intent
    Intent intent = getIntent();
    String action = intent.getStringExtra("action");

    // View Click Listener for registering on the buttons
    WSViewClickableListener viewClickListener = new WSViewClickableListener(this);

    if (action.equals("authoriseAccount")) {
      setContentView(R.layout.authorise_account);
      findViewById(R.id.button_authorise_account).setOnClickListener(viewClickListener);
    }
    else if (action.equals("changeUser")) {
      setContentView(R.layout.change_user);
      findViewById(R.id.button_change_user_authorise).setOnClickListener(viewClickListener);
      findViewById(R.id.button_change_user_cancel).setOnClickListener(viewClickListener);
    }
    else if (action.equals("updateCredentials")) {
      setContentView(R.layout.update_credentials);
      findViewById(R.id.button_update_credentials).setOnClickListener(viewClickListener);
    }

  }


  /***********************************************************************************************
  Resumes the <code>Activity</code> (captures the callback URL)
  ***********************************************************************************************/
  public void onNewIntent(Intent intent){

    if (intent == null) {
      // Hit the Back button while in the Webview. So, want to close the whole of the
      // "account generation" stuff and return to the "Accounts And Sync" menu
      finish();
      return;
    }

    super.onNewIntent(intent);

    Uri uri = intent.getData();
    if (uri != null) {

      // Get the code from the callback URL
      String code = uri.getQueryParameter("frob");
      // request a new access token for this application
      FlickrAuthenticator.retreiveAccessToken(code);

      // Return to the calling screen (FlickrUploader)
      setResult(RESULT_OK);
      finish();
    }

  }

}
