////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.send.flickr.authenticator;

import java.net.HttpURLConnection;
import java.net.URL;
import org.watto.android.component.WSWebView;
import org.watto.program.android.send.flickr.GlobalConstants;
import org.watto.program.android.send.flickr.security.FlickrAuthProtocol;
import android.content.Context;
import android.content.Intent;


/***********************************************************************************************
 Provides utility methods for communicating with the backend server
***********************************************************************************************/
public class FlickrAuthenticator {

  /** The Consumer Key, Consumer Secret, and callback URL for this application **/
  public static final FlickrAuthProtocol authClient = new FlickrAuthProtocol("120e1ac0b59ecc7805967db090f0024c","17429bd556c94d4c",GlobalConstants.CALLBACK_URL);


  /***********************************************************************************************
  Authenticates the <i>username</i> and <i>password</i> on the backend server
  ***********************************************************************************************/
  public static boolean confirmCredentials(){
    try {

      if (!credentialsKnown()) {
        return false;
      }

      // The URL is the details of "myself"
      URL authURL = new URL(authClient.signRequest("http://api.flickr.com/services/rest/?method=flickr.test.login"));
      HttpURLConnection request = (HttpURLConnection)authURL.openConnection();

      // Send the request
      request.connect();

      // Get the response code
      if (request.getResponseCode() == 200) {
        return true;
      }
      return false;

    }
    catch (Throwable t) {
      return false;
    }
  }


  /***********************************************************************************************
  Gets whether the token and secret are specified or not
  ***********************************************************************************************/
  public static boolean credentialsKnown(){
    String accessToken = authClient.getAccessToken();
    if (accessToken == null || accessToken.equals("")) {

      // Try to load the access token from the Settings
      authClient.loadFromSettings();

      // Re-check the access token
      accessToken = authClient.getAccessToken();
      if (accessToken == null || accessToken.equals("")) {
        return false;
      }
    }
    return true;
  }


  public static String getAccessTokenUrl(String pinCode){
    return authClient.getAccessTokenUrl(pinCode);
  }


  /***********************************************************************************************
  Connects the user to the authentication URL. The user signs in and approves this application for
  use with the backend server. The token and secrets are generated by the backend server, and
  stored locally for signing HTTP requests.
  ***********************************************************************************************/
  public static Intent getAuthenticationIntent(Context context){
    try {
      // Generate the authentication URL
      String authURL = authClient.getAuthenticationUrl();

      // Generate the "show in browser" Intent
      Intent intent = new Intent(context,WSWebView.class);
      intent.putExtra("url",authURL);
      return intent;
    }
    catch (Throwable t) {
      return null;
    }
  }


  public static void retreiveAccessToken(String code){
    authClient.retreiveAccessToken(code);
  }


  public static String signRequest(String url){
    return authClient.signRequest(url);
  }

}
