
package org.watto.program.android.send.flickr.http;

import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import org.watto.android.event.WSProgressInterface;


public class ListeningMultipartEntity extends MultipartEntity {

  private WSProgressInterface progressListener;
  private CountingOutputStream countingOutputStream;
  private OutputStream realOutputStream;


  public ListeningMultipartEntity(HttpMultipartMode mode, WSProgressInterface listener){
    super(mode);
    progressListener = listener;
  }


  public void writeTo(OutputStream out) throws IOException{
    // If we have yet to create the CountingOutputStream, or the
    // OutputStream being passed in is different from the OutputStream used
    // to create the current CountingOutputStream
    if ((realOutputStream == null) || (realOutputStream != out)) {
      realOutputStream = out;
      countingOutputStream = new CountingOutputStream(out);
    }

    super.writeTo(countingOutputStream);
  }

  private class CountingOutputStream extends FilterOutputStream {

    private long transferred = 0;
    private OutputStream wrappedOutputStream;


    public CountingOutputStream(final OutputStream out){
      super(out);
      wrappedOutputStream = out;
      transferred = 0;
    }


    public void write(byte[] b,int off,int len) throws IOException{
      wrappedOutputStream.write(b,off,len);
      transferred += len;
      progressListener.onProgressChanged(transferred);
    }


    public void write(int b) throws IOException{
      wrappedOutputStream.write(b);
      transferred++;
      progressListener.onProgressChanged(transferred);
    }
  }
}
